==> asu-0.6.6.tar.gz: asu-0.6.6/asu/common.py <==
import base64
import hashlib
import json
import struct
from pathlib import Path

import nacl.signing
import redis
import requests
from flask import current_app


def get_redis():
    return current_app.config["REDIS_CONN"]


def is_modified(url: str) -> bool:
    r = get_redis()

    modified_local = r.hget("last-modified", url)
    if modified_local:
        modified_local = modified_local.decode("utf-8")

    modified_remote = requests.head(url).headers.get("last-modified")

    if modified_local:
        if modified_local == modified_remote:
            return False

    if modified_remote:
        r.hset(
            "last-modified",
            url,
            modified_remote,
        )

    return True


def get_str_hash(string: str, length: int = 32) -> str:
    """Return sha256sum of str with optional length

    Args:
        string (str): input string
        length (int): hash length

    Returns:
        str: hash of string with specified length
    """
    h = hashlib.sha256()
    h.update(bytes(string, "utf-8"))
    response_hash = h.hexdigest()[:length]
    return response_hash


def get_file_hash(path: str) -> str:
    """Return sha256sum of given path

    Args:
        path (str): path to file

    Returns:
        str: hash of file
    """
    BLOCK_SIZE = 65536

    h = hashlib.sha256()
    with open(path, "rb") as f:
        fb = f.read(BLOCK_SIZE)
        while len(fb) > 0:
            h.update(fb)
            fb = f.read(BLOCK_SIZE)

    return h.hexdigest()


def get_manifest_hash(manifest: dict) -> str:
    """Return sha256sum of package manifest

    Duplicate packages are automatically removed and the list is sorted to be
    reproducible

    Args:
        manifest(dict): list of packages

    Returns:
        str: hash of `req`
    """
    return get_str_hash(json.dumps(manifest, sort_keys=True))


def get_request_hash(req: dict) -> str:
    """Return sha256sum of an image request

    Creates a reproducible hash of the request by sorting the arguments

    Args:
        req (dict): dict contianing request information

    Returns:
        str: hash of `req`
    """
    request_array = [
        req.get("distro", ""),
        req.get("version", ""),
        req.get("version_code", ""),
        req.get("profile", "").replace(",", "_"),
        get_packages_hash(req.get("packages", "")),
        get_manifest_hash(req.get("packages_versions", {})),
        str(req.get("diff_packages", False)),
    ]

    if "defaults" in req:
        request_array.append(get_str_hash(req.get("defaults", "")))

    return get_str_hash(" ".join(request_array), 32)


def get_packages_hash(packages: list) -> str:
    """Return sha256sum of package list

    Duplicate packages are automatically removed and the list is sorted to be
    reproducible

    Args:
        packages (list): list of packages

    Returns:
        str: hash of `req`
    """
    return get_str_hash(" ".join(sorted(list(set(packages)))), 12)


def fingerprint_pubkey_usign(pubkey: str) -> str:
    """Return fingerprint of signify/usign public key

    Args:
        pubkey (str): signify/usign public key

    Returns:
        str: string containing the fingerprint
    """
    keynum = base64.b64decode(pubkey.splitlines()[-1])[2:10]
    return "".join(format(x, "02x") for x in keynum)


def verify_usign(sig_file: Path, msg_file: Path, pub_key: str) -> bool:
    """Verify a signify/usign signature

    This implementation uses pynacl

    Args:
        sig_file (Path): signature file
        msg_file (Path): message file to be verified
        pub_key (str): public key to use for verification

    Returns:
        bool: Sucessfull verification

    Todo:
         Currently ignores keynum and pkalg

    """
    pkalg, keynum, pubkey = struct.unpack("!2s8s32s", base64.b64decode(pub_key))
    sig = base64.b64decode(sig_file.read_text().splitlines()[-1])

    pkalg, keynum, sig = struct.unpack("!2s8s64s", sig)

    verify_key = nacl.signing.VerifyKey(pubkey, encoder=nacl.encoding.RawEncoder)
    try:
        verify_key.verify(msg_file.read_bytes(), sig)
        return True
    except nacl.exceptions.CryptoError:
        return False


def stats_versions():
    return [
        (int(s), p.decode("utf-8"))
        for p, s in get_redis().zrevrange(f"stats-versions", 0, -1, withscores=True)
    ]


def stats_profiles(branch):
    return [
        (int(s), p.decode("utf-8"))
        for p, s in get_redis().zrevrange(
            f"stats-profiles-{branch}", 0, -1, withscores=True
        )
    ]

==> asu-0.7.17.tar.gz: asu-0.7.17/asu/common.py <==
import base64
import hashlib
import json
import struct
from pathlib import Path

import nacl.signing
import requests
from flask import current_app


def get_redis():
    return current_app.config["REDIS_CONN"]


def is_modified(url: str) -> bool:
    r = get_redis()

    modified_local = r.hget("last-modified", url)
    if modified_local:
        modified_local = modified_local.decode("utf-8")

    modified_remote = requests.head(url).headers.get("last-modified")

    if modified_local:
        if modified_local == modified_remote:
            return False

    if modified_remote:
        r.hset(
            "last-modified",
            url,
            modified_remote,
        )

    return True


def get_str_hash(string: str, length: int = 32) -> str:
    """Return sha256sum of str with optional length

    Args:
        string (str): input string
        length (int): hash length

    Returns:
        str: hash of string with specified length
    """
    h = hashlib.sha256()
    h.update(bytes(string, "utf-8"))
    response_hash = h.hexdigest()[:length]
    return response_hash


def get_file_hash(path: str) -> str:
    """Return sha256sum of given path

    Args:
        path (str): path to file

    Returns:
        str: hash of file
    """
    BLOCK_SIZE = 65536

    h = hashlib.sha256()
    with open(path, "rb") as f:
        fb = f.read(BLOCK_SIZE)
        while len(fb) > 0:
            h.update(fb)
            fb = f.read(BLOCK_SIZE)

    return h.hexdigest()


def get_manifest_hash(manifest: dict) -> str:
    """Return sha256sum of package manifest

    Duplicate packages are automatically removed and the list is sorted to be
    reproducible

    Args:
        manifest(dict): list of packages

    Returns:
        str: hash of `req`
    """
    return get_str_hash(json.dumps(manifest, sort_keys=True))


def get_request_hash(req: dict) -> str:
    """Return sha256sum of an image request

    Creates a reproducible hash of the request by sorting the arguments

    Args:
        req (dict): dict contianing request information

    Returns:
        str: hash of `req`
    """
    request_array = [
        req.get("distro", ""),
        req.get("version", ""),
        req.get("version_code", ""),
        req.get("profile", "").replace(",", "_"),
        get_packages_hash(req.get("packages", "")),
        get_manifest_hash(req.get("packages_versions", {})),
        str(req.get("diff_packages", False)),
        req.get("filesystem", ""),
    ]

    if "defaults" in req:
        request_array.append(get_str_hash(req.get("defaults", "")))

    return get_str_hash(" ".join(request_array), 32)


def get_packages_hash(packages: list) -> str:
    """Return sha256sum of package list

    Duplicate packages are automatically removed and the list is sorted to be
    reproducible

    Args:
        packages (list): list of packages

    Returns:
        str: hash of `req`
    """
    return get_str_hash(" ".join(sorted(list(set(packages)))), 12)


def fingerprint_pubkey_usign(pubkey: str) -> str:
    """Return fingerprint of signify/usign public key

    Args:
        pubkey (str): signify/usign public key

    Returns:
        str: string containing the fingerprint
    """
    keynum = base64.b64decode(pubkey.splitlines()[-1])[2:10]
    return "".join(format(x, "02x") for x in keynum)


def verify_usign(sig_file: Path, msg_file: Path, pub_key: str) -> bool:
    """Verify a signify/usign signature

    This implementation uses pynacl

    Args:
        sig_file (Path): signature file
        msg_file (Path): message file to be verified
        pub_key (str): public key to use for verification

    Returns:
        bool: Sucessfull verification

    Todo:
         Currently ignores keynum and pkalg

    """
    pkalg, keynum, pubkey = struct.unpack("!2s8s32s", base64.b64decode(pub_key))
    sig = base64.b64decode(sig_file.read_text().splitlines()[-1])

    pkalg, keynum, sig = struct.unpack("!2s8s64s", sig)

    verify_key = nacl.signing.VerifyKey(pubkey, encoder=nacl.encoding.RawEncoder)
    try:
        verify_key.verify(msg_file.read_bytes(), sig)
        return True
    except nacl.exceptions.CryptoError:
        return False
